;######################################################################
;Smooth Scrolling Tile Map Routine
;######################################################################
;X Cord -> BC
;Y Cord -> DE
;Tile Pointer -> HL
;Map Pointer -> IX
	
SmoothScrl:
	di
	ld a,$80
	out ($10),a
	ld (Tile_Pointer),hl
	ld a,(ix)
	ld (Map_Width),a
	srl b
	rr c
	rr a
	srl b
	rr c
	rr a
	srl b
	rr c
	rr a
	srl a
	srl a
	srl a
	srl a 
	srl a
	ld (X_Offset),a
	srl d
	rr e
	rr a
	srl d
	rr e
	rr a
	srl d
	rr e
	rr a
	srl a
	srl a
	srl a
	srl a
	srl a
	ld (Y_Offset),a
	ld hl,0
	ld a,e
	cp 0
	jp z,SkipRowJump
	ld b,e
	ld de,(Map_Width)
	ld d,0
YLoop:
	add hl,de
	djnz YLoop
SkipRowJump:
	add hl,bc                            ;finds map offset
	ld (Map_Pointer),hl
	ld de,(Map_Pointer)                         ;gets hl into de
	add ix,de                            ;adds map offset to map pointer
	ld a,(X_Offset)
	cp 4
	jp m,ALeft                           ;checks to see if it should align left or right
	ld (Map_Pointer),ix
	ld de,3
	add ix,de
	jp BlitTiles
ALeft:
	ld de,2
	add ix,de
	ld (Map_Pointer),ix
	
;*************************************************
;Blit Tiles
;*************************************************
;c is x offset
;a is y offset
;ix is map pointer
;(Map_Pointer) is map pointer save
;(Map_Width) is width of map

BlitTiles:
	ld hl,Plotsscreen       ;10   
	ld a,(Y_Offset)
	cp 0                    ;7   
	ld c,$20   
	jp z,VertAligned        ;10/1
	
	ld b,12                 ;7
BlitColumn:
	push ix                 ;15
	push hl                 ;11
	push bc                 ;11
	
	ex af,af'
	ld a,c
	out ($10),a
	ex af,af'
	
	ld d,0                               ;clear d
	ld e,(ix)                            ;load tile # from map
	ld iy,(Tile_Pointer)                   ;load starting point into hl
	sla e                                ;e * 8
	sla e
	sla e
	add iy,de                            ;get to correct tile
	ld d,0                               ;clear d
	ld e,a                               ;load e as the offset (a)
	add iy,de                            ;add offset
	
	sub 8
	neg
	ld b,a
	call Blit
	
	ld de,(Map_Width)
	ld d,0
	add ix,de
	
	ld b,7
BlitMiddleRows:
	push bc
	
	ld d,0
	ld e,(ix)
	ld iy,(Tile_Pointer)
	sla e 
	sla e
	sla e
	add iy,de
	
	ld b,8
	call Blit
	
	ld de,(Map_Width)
	ld d,0
	add ix,de
	
	pop bc
	djnz BlitMiddleRows
	 
	ld d,0
	ld e,(ix)
	ld iy,(Tile_Pointer)
	sla e 
	sla e
	sla e
	add iy,de
	
	sub 8
	neg
	ld b,a
	call Blit
	
	pop bc
	inc c
	pop hl
	inc hl
	pop ix
	inc ix
	djnz BlitColumn
	
	jp RoutTransit
	
VertAligned:
	ld b,12

VertLoop:
	push ix
	push hl
	push bc
	
	ex af,af'
	ld a,c
	out ($10),a
	ex af,af'
	
	ld b,8
BlitVertLoop:
	push bc
	
	ld d,0
	ld e,(ix)
	ld iy,(Tile_Pointer)
	sla e 
	sla e
	sla e
	add iy,de
	
	ld b,8
	call Blit
	
	ld de,(Map_Width)
	ld d,0
	add ix,de
	
	pop bc
	djnz BlitVertLoop
	
	pop bc
	inc c
	pop hl
	inc hl
	pop ix
	inc ix
	djnz VertLoop

;*************************************************
RoutTransit:
	ld a,(X_Offset)                               ;see if we should scroll left or right
	cp 0
	jp z,Finish
	cp 4
	jp m,Sleftbgn
	sub 8
	neg
	ld c,a
	ld a,(Y_Offset)
	
;*************************************************
;Shift Tiles
;*************************************************
;c is x offset
;a is y offset
;ix is map pointer
	
	ld hl,plotsscreen                    ;plotsscreen pointer to hl
	ld de,(Map_Width)
	ld d,0
	ld (SaveNumCol1_SMC),de
	ld ix,(Map_Pointer)
	inc ix 
	inc ix
	ld b,64                              ;64 rows to scroll
Sright:
    ld d,0                               ;clear d
	ld e,(ix)                            ;load tile # from map
	ld (SaveMapPos1_SMC),ix
	ld ix,(Tile_Pointer)                   ;load starting point into hl
	sla e                                ;c * 8
	sla e
	sla e
	add ix,de                            ;get to correct tile
	ld d,0                               ;clear d
	ld e,a                               ;load e as the y offset (a)
	add ix,de                            ;add offset
	ld e,(ix)
Scrollrr:
	push bc
	ld b,c                               ;ld x offset into b
	ld (SavePS1_SMC),hl
Scrollr:
	srl e
	rr (hl)
	inc hl
	rr (hl)
	inc hl	
	rr (hl)
	inc hl	
	rr (hl)
	inc hl	
	rr (hl)
	inc hl	
	rr (hl)
	inc hl	
	rr (hl)
	inc hl	
	rr (hl)
	inc hl	
	rr (hl)
	inc hl	
	rr (hl)
	inc hl	
	rr (hl)
	inc hl	
	rr (hl)
	inc hl
SavePS1_SMC = $+1
	ld hl,$0000
	djnz Scrollr
	
	ld de,12
	add hl,de
	
	inc a                                ;inc offset
	and %00000111                        ;mask the offset so that it will nvr be above 7
	cp 0                                 ;chk if the Map row needs to be changed
	jp nz,SkipRowChgSr                    

SaveNumCol1_SMC = $+1	
	ld de,$0000                   ;if so then add the # columns to the  pointer
SaveMapPos1_SMC = $+2
	ld ix,$0000
	add ix,de
	pop bc
	djnz Sright
	
	jp Finish
SkipRowChgSr:	
	inc ix 
	ld e,(ix)
	pop bc
	djnz Scrollrr
	
    jp Finish
	
Sleftbgn:
	ld c,a
	ld a,(Y_Offset)
	ld hl,plotsscreen+11
	ld de,(Map_Width)
	ld d,0
	ld (SaveNumCol2_SMC),de
	ld ix,(Map_Pointer)
	ld de,12
	add ix,de
	ld b,64
Sleft:
    ld d,0                               ;clear d
	ld e,(ix)                            ;load tile # from map
	ld (SaveMapPos2_SMC),ix
	ld ix,(Tile_Pointer)                   ;load starting point into hl
	sla e                                ;c * 8
	sla e
	sla e
	add ix,de                            ;get to correct tile
	ld d,0                               ;clear d
	ld e,a                               ;load e as the y offset (a)
	add ix,de                            ;add offset
	ld e,(ix)
Scrollll:
	push bc
	ld b,c
	ld (SavePS2_SMC),hl
Scrolll:
	sla e
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
SavePS2_SMC = $+1
	ld hl,$0000
	djnz Scrolll
	
	ld de,12
	add hl,de
	
	inc a                                ;inc offset
	and %00000111                        ;mask the offset so that it will nvr be above 7
	cp 0                                 ;chk if the Map row needs to be changed
	jp nz,SkipRowChgLr                    

SaveNumCol2_SMC = $+1	
	ld de,$0000                          ;if so then add the # columns to the  pointer
SaveMapPos2_SMC = $+2
	ld ix,$0000
	add ix,de
	pop bc
	djnz Sleft
	
	jp Finish
SkipRowChgLr:	
	inc ix 
	ld e,(ix)	
	pop bc
	djnz Scrollll

Finish:
	ld iy,Flags
	ei
	ret

	
Blit:
	ex af,af'           ;4
BlitLoop:
	ld a,(hl)           ;7
	out ($11),a         ;11
	ld a,(iy)           ;19
	ld (hl),a           ;7
	ld de,12            ;10
	add hl,de           ;11
	inc iy              ;23
	djnz BlitLoop       ;13/8          101/loop
	ex af,af'           ;4
	ret                 ;10            
	
X_Offset:
	.db $00
Y_Offset:
	.db $00
Map_Width:
	.db $00	
Map_Pointer:
	.dw $0000
Tile_Pointer:
	.dw $0000
	
;*************************************************
;#################################################
.end
